classdef BayesClassifier
%-------------------------------------------------------------------------
%Jan Hrdlicka 21.11.2010
%-------------------------
% Class of the Bayes classifier objects, 
% For now, it is suitable only for continuous input, 2 classes and it expects normal
% distribution of the data

    properties
        gaussMean = []; %with size CxD, C: number of classes, D: number of dimensions
        gaussVar = []; %with size CxD, C: number of classes, D: number of dimensions
        numDim = 0;
        prior = []; %prior probabilities
        continuousInput = true;
        numClass = 2; %number of the classes
        trained = false; %is bayes already trained?
        costs = ones(1,2);
    end
    methods
        function obj = BayesClassifier(varargin)
            %constructor. For now, only one argument is possible- flag of
            %the continuous input
            if nargin==0
                obj.continuousInput = true;
            elseif nargin==1
                obj.continuousInput = varargin{1};
            else
                display('BayesClassifier: too many input arguments');
            end
        end

        function obj = setCosts(obj, costs)
            %expects input 'costs' in vector [FNcost FPcost] (TP and TN costs are assumed zero)
            if size(costs)==[1,2]
                obj.costs = costs;
            else
                display('Cost is expected to be vector of length 2');
            end
        end
            
        function obj = train(obj,trainData)
            % train Bayes on the trainData (find the mean and variance of the gaussians)
            obj.numDim = size(trainData.samples,2);
            if obj.continuousInput
                posSamples = trainData.samples(trainData.class==1,:);
                negSamples = trainData.samples(trainData.class==0,:);
                obj.gaussMean(1,:) = nanmean(posSamples,1);
                obj.gaussVar(1,:) = nanvar(posSamples,1);
                obj.gaussMean(2,:) = nanmean(negSamples,1);
                obj.gaussVar(2,:) = nanvar(negSamples,1);
                obj.prior(1,1) = size(posSamples,1)/(size(posSamples,1)+size(negSamples,1));
                obj.prior(2,1) = size(negSamples,1)/(size(posSamples,1)+size(negSamples,1));
            else
                display('BayesClasssifier.train: Discrete input not implemented');
            end
            obj.trained = true;
        end
        
        function ratio = classify(obj,data)
            %returns the ratio
            if obj.trained
                for i = 1:obj.numDim
                    p1(i,:) = normpdf(data(:,i),obj.gaussMean(1,i),obj.gaussVar(1,i));
                    p2(i,:) = normpdf(data(:,i),obj.gaussMean(2,i),obj.gaussVar(2,i));
                end
                ratio = (sum(log(p1),1)-sum(log(p2),1));
            else
                display('BayesClassifier.classify: Bayes classifier is not trained');
            end
        end
        
    end
end